package domain

type EngagementService interface {
	ViewRecipe(recipeId int) (Engagement, error)
	ShareRecipe(recipeId int) (Engagement, error)
	UserViewRecipe(userId, recipeId int) (Engagement, error)
	UserFavoriteRecipe(userId, recipeId int) (Engagement, error)
	UserMakeRecipe(userId, recipeId int) (Engagement, error)
	UserShareRecipe(userId, recipeId int) (Engagement, error)
	UserCreateRecipe(userId, recipeId int) (Engagement, error)
	UserDeleteRecipe(userId, recipeId int) (Engagement, error)
	UserEditRecipe(userId, recipeId int) (Engagement, error)
	GetUserEngagement(userId, limit int) ([]Engagement, error)
}
